#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>

#include <ctype.h>

#define NEWLINE 	1	/* '\n' */
#define STRING		2	/* */
#define NUMBER 		3	/* numeric constant */
#define LETTER 		4	/* 'a'..'z', 'A'..'Z' */
#define ASSIGN_OP	5	/* '=', '+=', '-=', '*=', '/=', '%=', '^=' */
#define MUL_OP 		6	/* '*', '/', '%' */
#define REL_OP 		7	/* '==', '<=', '>=', '!=', '<', '>'*/
#define INCR_DECR 	8	/* '++', '--' */
#define EOF 		0	/* the end of input */	
#define COMMA		9	/* ',' */
#define OPENBRACKET	10	/* '(' */
#define CLOSEBRACKET	11	/* ')' */
#define PLUS		12	/* '+' */
#define MINUS		13	/* '-' */
#define DEGREE		14	/* '^' */
#define SQUAREOB	15	/* '[' */
#define SQUARECB	16	/* ']' */
#define CURLYOB		17	/* '{' */
#define CURLYCB		18	/* '}' */
#define SEMICOLON	19	/* ';' */
#define VARIABLE	20
#define Auto		21
#define Ibase		22
#define Length		23
#define Return		24
#define While		25
#define Break		26
#define If		27
#define Obase		28
#define Scale		29
#define Define		30
#define For		31
#define Quit		32
#define Sqrt		33

double TOKVAL;
char *buf;
int len = 0;
int i;
int quit_val = 0;
int c;

struct keywords {
	char *name;
	int val;
};

struct keywords tbl[] = {
	"auto", 21,
	"ibase", 22,
	"length", 23,
	"return", 24,
	"while", 25,
	"break", 26,
	"if", 27,
	"obase", 28,
	"scale", 29,
	"define", 30,
	"for", 31,
	"quit", 32,
	"sqrt", 33,
	0, 0
};


int getop()
{
	buf = (char *) malloc(sizeof(buf));
	i = 0;
		
	switch (c) {
		case ',':
			return COMMA;
		case '[':
			return SQUAREOB;
		case ']':
			return SQUARECB;
		case '{':
			return CURLYOB;
		case '}':
			return CURLYCB;
		case '(':
			return OPENBRACKET;
		case ')':
			return CLOSEBRACKET;
		case '+':
			buf[i++] = c;
			c = getchar(); 
				
			if (c == '+') {
				buf[i++] = c;
				buf[i] = 0;
				return INCR_DECR;
			} else if (c == '=') {
				buf[i++] = c;
				buf[i] = 0;
				return ASSIGN_OP;
			}
				
			ungetc (c, stdin);
			return PLUS;
		case '-':
			buf[i++] = c;
			c = getchar(); 
				
			if (c == '-') {
				buf[i++] = c;
				buf[i] = 0;
				return INCR_DECR;
			} else if (c == '=') {
				buf[i++] = c;
				buf[i] = 0;
				return ASSIGN_OP;
			}
				
			ungetc(c, stdin);
			return MINUS;
		case '/':
		case '%':
		case '*':
			buf[i++] = c;
			c = getchar(); 
				
			if (c == '=') {
				buf[i++] = c;
				buf[i] = 0;
				return ASSIGN_OP;
			}
				
			ungetc(c, stdin);
			return MUL_OP;
		case '^':
			buf[i++] = c;
			c = getchar(); 
				
			if (c == '=') {
				buf[i++] = c;
				buf[i] = 0;
				return ASSIGN_OP;
			}
				
			ungetc(c, stdin);
			return DEGREE;
		case '<':
		case '>':
			buf[i++] = c;
			c = getchar(); 
				
			if (c == '=') {
				buf[i++] = c;
				buf[i] = 0;
				return REL_OP;
			}
				
			ungetc(c, stdin);
			return REL_OP;
		case '=':
			buf[i++] = c;
			c = getchar(); 
				
			if (c == '=') {
				buf[i++] = c;
				buf[i] = 0;
				return REL_OP;
			}
				
			ungetc(c, stdin);
			return ASSIGN_OP;
		default:
			break;
	}
}

void getn()
{
	ungetc (c, stdin);
	scanf ("%lf", &TOKVAL);
}
	
void getf()
{	
	if ((buf = (char *) malloc(sizeof(buf))) == NULL) {
		printf("Enough memory\n");
		exit(1);
	} else {
		i = 0;
			
		do {	
			buf[i++] = c;
			c = getchar ();
		} while ((c != EOF && isalnum (c)) || (c != EOF && c == '_'));
		
		buf[i] = 0;		
		ungetc (c, stdin);
	}
}

int get_next_token (void)
{
	buf = NULL;
	
	while ((c = getchar ()) == ' ' || c == '\t');

	if (c == EOF)
		return EOF;
		
	if (c == '\n')
		return NEWLINE;
	
	if (strchr("=+-*/^()<>!%,{}[];", c) != NULL)
		getop();
	

	if (c == '.' || isdigit (c)) {
		getn();
		return NUMBER;
	}

	if (isalpha (c)) {
		getf();
		
		if (i == 1)
			return LETTER;
		
		int k;
		
		for (k = 0; tbl[k].name != 0; k++)
			if (strcmp(buf, tbl[k].name) == 0)
				return tbl[k].val;
				
		return VARIABLE;
	}
	
	
	if (c == '"') {
		c = getchar();
		
		if ((buf = (char *)malloc (sizeof(buf))) == NULL) {
			printf("Enough memory\n");
			exit(1);
		} else {
			i = 0;
			
			if (c == '"') {
				c = getchar();
				ungetc(c, stdin);
			} else if (c == '\\') {
				c = getchar();
				c = getchar();
				
				if (c == '"')
					c = getchar();
					
				ungetc(c, stdin);
			} else {
				do {
					buf[i++] = c;
					c = getchar ();
				} while ((c != '"') && (c != '\\'));
				
				if (c == '"')
					c = getchar();
					
				ungetc(c, stdin);
			}
			
			buf[i] = 0;
			return STRING;
		}
	}
	
	if (c == '\\') {
		c = getchar();
		c = getchar();
		
		if ((buf = (char *)malloc (sizeof(buf) + 1)) == NULL) {
			printf("Enough memory\n");
			exit(1);
		} else {
			i = 0;
			
			if (c == '"') {
				c = getchar();
				ungetc(c, stdin);
			} else if (c == '\\') {
				c = getchar();
				c = getchar();
				
				if (c == '"')
					c = getchar();
					
				ungetc(c, stdin);
			} else {
				do {
					buf[i++] = c;
					c = getchar ();
				} while ((c != '"') && (c != '\\'));

				if (c == '"')
					c = getchar();

				ungetc(c, stdin);
			}
			
			buf[i] = 0;
			return STRING;
		}
	}
}

int main(int argc, char *argv[])
{
	
	while (1) {
		int t = get_next_token();
		if (t == Quit) break;
	}
	
	free(buf);
	return EXIT_SUCCESS;
}